function Osmosis
% Notes and modifications:
% - Hardcoded path to file needs to changed as needed (line 128).
% - Simulation set for Model B of Waniewski et al. 2009 paper.
% END - Notes and modifications - August 2010
%------------------------------------
%extended darcy law with osmotic effect

%choice of osmotic agent 
%choise of theta: constant or depend on P

% units : cm, min, mmol/mL
%------------------------------------
clear all;
close all;

tic
%----------------FUGURES SWITCH--------------------------------
fig=1; %switch for figures 1 - on, 0 - off
%-----------------STEADY STATES-------------------------------
SteadyState=0;  %to switch off calculateions of steady states write 0 -off, 1 - on
%----------------------------------------------------------
m = 0;
PIP=[2 5.56 7 12 0.5 8 3 1 20 30];
profil=3;
P0=0;
Pd=PIP(profil)
%-----------------------------X AND T SPACE---------------------

xmax=1;
xnumb=1001;

x=linspace(0,xmax,xnumb);

tnumb=1001;
tmax=100;
t = linspace(0,tmax,tnumb);
%--------------------OSMOTIC AGENT------------------------
OsmoticAgent={'Glucose';'Icodextrin 7.5%';'Mannitol 5%'};
solute=OsmoticAgent(1)  %choose the name of the osmotic agent
%---------------------VOID VOLUME CHOICE--CONST or DEPEND ON P-------------
ThetaChoice=1    
% if constant write 0
% if depend on P write 1
%-----------------------PARAMETERS-----------------------------
[Cb,Cd]=OsmAgentParam(solute); 
param1=1

%pa=0.034; %1/min                %Old kijow
pa=0.047 %1/min                 %New Kijow MF
% pa=0.0714; %1/min               %Watson
% pa=0.1213/param1                %MF Mannitol
% pa=0.067;                        % model "rat"

%Lpa=7.3*10^(-5) %1/min/mmHg     %Kijow
% Lpa=14.835*10^(-5) %1/min/mmHg  %Watson
Lpa=36.6*10^(-5) %1/min/mmHg     %MF  +  model "rat"

%sigmaOsmAg=0.020       %    0.0235     % model  "rat"
sigmaOsmAg=0.16        % 0.15        %0.21                    %MF fit
% sigmaOsmAg=0.46     % 0.6117                 %Watson

%tissue parameters are loaded inside the functions
RT=19.3*10^(3);    %mmHg*mL/mmol
C0=Cb;

AA=0.6*10^4 %tissue surface for exchange cm^2

%----------------------MAIN PROGRAM--------------------------
%options=odeset('RelTol',1e-7,'AbsTol',1e-8);
sol = pdepe(m,@peq,@pic,@pbc,x,t);

P=sol(:,:,1);
C=sol(:,:,2);

pout=ones(tnumb,xnumb);
dpout=ones(tnumb,xnumb);
cout=ones(tnumb,xnumb);
dcout=ones(tnumb,xnumb);

for j=1:tnumb
[pout(j,:),dpout(j,:)] = pdeval(m,x,P(j,:),x);
[cout(j,:),dcout(j,:)] = pdeval(m,x,C(j,:),x);
end;
%---------------------------------Figures
if fig==1
Figures(tnumb,tmax,xnumb,xmax,pout,dpout,cout,dcout,P0,ThetaChoice,solute,RT,Cd,C0,param1,AA)
end;

[theta,thetaSwitch,sigmaT,Keff,Deff]=TissueParam(pout,P0,ThetaChoice,solute,param1);

Kth=0.139*10^(-4)
LambdaF=sqrt(Kth/Lpa)
LambdaS=0.0147;  %   0.016
LambdaSdiff=0.021
fi=xmax/LambdaF
alfa=LambdaF/LambdaS

phydr1=P0+(Pd-P0)*cosh(fi*(1-x/xmax))/cosh(fi);
phydr2=Pd-(Pd-P0)*(1-exp(-x/LambdaS))-RT*(Cd-C0)*sigmaOsmAg*x*exp(-xmax/LambdaS)/alfa/alfa/LambdaS;
Cappr=(C0+(Cd-Cb)*exp(-x/LambdaS))/Cd;
Cdif=(C0+(Cd-Cb)*exp(-x/LambdaSdiff))/Cd;

fonts=12;
figure
plot(x,pout(tnumb,:),'-',x,phydr2,'--',x,phydr1,':','LineWidth',2)
xlabel('Distance X, cm','FontWeight','bold','FontSize',fonts);
ylabel('Interstitial hydrostatic presure P, mmHg','FontWeight','bold','FontSize',fonts);
legend('Simulated','Calculated','Calculated for hydrostatic absorption');

figure
plot(x,cout(tnumb,:)/Cd,'-',x,Cappr,'--',x,Cdif,':','LineWidth',2)
xlabel('Distance X, cm','FontWeight','bold','FontSize',fonts);
ylabel('C/Cd','FontWeight','bold','FontSize',fonts);
legend('Simulated','Exponential approximation','Calculated for pure diffusion');

%---------------------------------OUTPUT--------------
sigmaT
jv=Keff.*(-dpout + sigmaT.*RT.*dcout);
Pendend=pout(tnumb,xnumb)  %mmHg
jv0=jv(tnumb,1)  %cm/min
JvIN=jv(tnumb,1)*AA %mL/min
%----------------------------------SAVE----------------
Cnorm=(cout(tnumb,:)-Cb)./(Cd-Cb);
ChumanMF=Cnorm';
% Please put in appropriate file path:
%save('C:\Matlab7\Osmosis_clinical\ChumanMF.txt','ChumanMF','-ASCII');
save('~/Osmosis_clinical/ChumanMF.txt','ChumanMF','-ASCII');
save('ChumanMF','x','Cnorm');
%------------------------------STEADY STATES--------------------
if SteadyState==1
i=1;
timeP=i;
for j=1:xnumb
distance(i,j)=abs(pout(i,j)-pout(tnumb,j));
end;
[maxdist(i),maxx(i)]=max(distance(i,:));

while (maxdist(i)>0.1*pout(tnumb,maxx(i))) & (i<=tnumb)
       i=i+1;
       for j=1:xnumb
            distance(i,j)=abs(pout(i,j)-pout(tnumb,j));
       end;
       [maxdist(i),maxx(i)]=max(distance(i,:));
       timeP=i+1;
end;
if timeP>tnumb
    disp('Steady state for P is not obtained');
else
    tt=timeP*tmax/((tnumb-1)*60);
    disp(['Steady state for P is obtained after ', num2str(tt),' minutes']);
end;
%--------------------Tsteady for JvIN-----
for z=2:tnumb
    if (abs(jv(z,1)-jv(tnumb,1)) < abs(0.1*jv(tnumb,1)))
        if (abs(jv(z-1,1)-jv(tnumb,1)) > abs(0.1*jv(tnumb,1)))
            TsteadyS=z*tmax/(tnumb-1);
            TsteadyMin=TsteadyS/60;
        end;
    end;
end;
disp(['Steady state for JvIN is obtained after ', num2str(TsteadyMin),' minutes']);
%---------------------Tsteady for C for osmotic agent---
i=1;
timeC=i;
for j=1:xnumb
distance(i,j)=abs(cout(i,j)-cout(tnumb,j));
end;
[maxdist(i),maxx(i)]=max(distance(i,:));

while (maxdist(i)>0.1*cout(tnumb,maxx(i))) & (i<=tnumb)
       i=i+1;
       for j=1:xnumb
            distance(i,j)=abs(cout(i,j)-cout(tnumb,j));
       end;
       [maxdist(i),maxx(i)]=max(distance(i,:));
       timeC=i+1;
end;
if timeC>tnumb
    disp('Steady state for C is not obtained');
else
    tt=timeC*tmax/((tnumb-1)*60);
    disp(['Steady state for C is obtained after ', num2str(tt),' minutes']);
end;
end;




%----------------------------------------------------------
beep
toc

%-------------------FUNCTIONS------------

function [c,f,s] = peq(x,t,u,DuDx);

[theta,thetaSwitch,sigmaT,Keff,Deff]=TissueParam(u(1),P0,ThetaChoice,solute,param1);
    
jv=Keff*(-DuDx(1)+sigmaT*RT*DuDx(2));

qv=-Lpa*sigmaOsmAg*RT*(Cb-u(2));
f=0.5;
qs=pa*(Cb-u(2))+(1-sigmaOsmAg)*qv*((1-f)*Cb+f*u(2));
js=-Deff*DuDx(2)+(1-sigmaT)*jv*u(2);

c1 = thetaSwitch;
f1 = -jv;
s1 = qv;
c2 = theta;
f2 = -js + u(2)*jv;
s2 = qs - jv* DuDx(2) - qv*u(2);
c = [c1; c2];
f = [f1; f2];
s = [s1; s2];
end
%---------------------------------------------------------
function u0 = pic(x);
u0 = [0; C0];
end
%-------------------------------------------------------
function [pmin,qmin,pmax,qmax] = pbc(xmin,umin,xmax,umax,t);
pmin = [umin(1) - Pd; umin(2)-Cd];
qmin = [0; 0];
pmax = [0; 0];
qmax = [1; 1];
end
end
